@@ -11,22 +11,28 @@ esac
 echo
 echo "Building ugrep..."
 
-# fix git clone timestamp issues causing build failures
-touch config.h.in lib/Makefile.in src/Makefile.in
-
-# configure with colors enabled by default or the command arguments
-OPTIONS=${1:---enable-color}
+# configure with the specified command arguments
+OPTIONS=$@
 
 echo
 echo "./configure $OPTIONS"
 echo
 
+# appease automake when the original timestamps are lost, when using git clone
+touch aclocal.m4 Makefile.am lib/Makefile.am src/Makefile.am
+sleep 1
+touch config.h.in Makefile.in lib/Makefile.in src/Makefile.in
+sleep 1
+touch configure
+
 ./configure $OPTIONS
 
 echo
 echo "make -j clean all"
 echo
 
+make clean
+
 if ! make -j ; then
 echo "Failed to build ugrep: please run the following two commands:"
 echo "$ autoreconf -fi"
