@@ -101,7 +101,7 @@ class AbstractMatcher {
     static const int UNK      = 256;        ///< unknown/undefined character meta-char marker
     static const int BOB      = 257;        ///< begin of buffer meta-char marker
     static const int EOB      = EOF;        ///< end of buffer meta-char marker
-    static const size_t BLOCK = 8192;       ///< buffer growth factor, buffer is initially 2*BLOCK size, at least 256
+    static const size_t BLOCK = (256*1024); ///< buffer size and growth, buffer is initially 2*BLOCK size, at least 4096 bytes
     static const size_t REDO  = 0x7FFFFFFF; ///< reflex::Matcher::accept() returns "redo" with reflex::Matcher option "A"
     static const size_t EMPTY = 0xFFFFFFFF; ///< accept() returns "empty" last split at end of input
   };
@@ -299,7 +299,11 @@ class AbstractMatcher {
     if (own_)
     {
 #if defined(WITH_REALLOC)
+#if (defined(__WIN32__) || defined(_WIN32) || defined(WIN32) || defined(_WIN64) || defined(__BORLANDC__)) && !defined(__CYGWIN__) && !defined(__MINGW32__) && !defined(__MINGW64__)
+      _aligned_free(static_cast<void*>(buf_));
+#else
       std::free(static_cast<void*>(buf_));
+#endif
 #else
       delete[] buf_;
 #endif
@@ -337,10 +341,19 @@ class AbstractMatcher {
     }
     if (!own_)
     {
+      max_ = 2 * Const::BLOCK;
 #if defined(WITH_REALLOC)
-      buf_ = static_cast<char*>(std::malloc(max_ = 2 * Const::BLOCK));
+#if (defined(__WIN32__) || defined(_WIN32) || defined(WIN32) || defined(_WIN64) || defined(__BORLANDC__)) && !defined(__CYGWIN__) && !defined(__MINGW32__) && !defined(__MINGW64__)
+      buf_ = static_cast<char*>(_aligned_malloc(max_, 4096));
+      if (buf_ == NULL)
+        throw std::bad_alloc();
 #else
-      buf_ = new char[max_ = 2 * Const::BLOCK];
+      buf_ = NULL;
+      if (::posix_memalign(reinterpret_cast<void**>(&buf_), 4096, max_) != 0)
+        throw std::bad_alloc();
+#endif
+#else
+      buf_ = new char[max_];
 #endif
     }
     buf_[0] = '\0';
@@ -368,21 +381,17 @@ class AbstractMatcher {
     eof_ = false;
     mat_ = false;
   }
-  /// Set buffer block size for reading: use 1 for interactive input, 0 (or omit argument) to buffer all input in which case returns true if all the data could be read and false if a read error occurred.
+  /// Set buffer block size for reading: use 0 (or omit argument) to buffer all input in which case returns true if all the data could be read and false if a read error occurred.
   bool buffer(size_t blk = 0) ///< new block size between 1 and Const::BLOCK, or 0 to buffer all input (default)
     /// @returns true when successful to buffer all input when n=0
   {
-    if (eof_)
-      return true;
     if (blk > Const::BLOCK)
       blk = Const::BLOCK;
     DBGLOG("AbstractMatcher::buffer(%zu)", blk);
     blk_ = blk;
-    if (blk > 0)
+    if (blk > 0 || eof_ || in.eof())
       return true;
-    if (in.eof())
-      return true;
-    size_t n = in.size(); // get the (rest of the) data size, which is 0 if unknown (e.g. TTY)
+    size_t n = in.size(); // get the (rest of the) data size, which is 0 if unknown (e.g. reading input from a TTY or a pipe)
     if (n > 0)
     {
       (void)grow(n + 1); // now attempt to fetch all (remaining) data to store in the buffer, +1 for a final \0
@@ -404,18 +413,18 @@ class AbstractMatcher {
     evh_ = handler;
   }
   /// Get the buffered context before the matching line.
-  Context before()
+  inline Context before()
   {
     (void)lineno();
     return Context(buf_, bol_ - buf_, num_);
   }
   /// Get the buffered context after EOF is reached.
-  Context after()
+  inline Context after()
   {
     if (hit_end())
     {
       (void)lineno();
-      // if there is no \n at the end of input: increase line count by one
+      // if there is no \n at the end of input: increase line count by one to compensate
       if (bol_ < txt_)
         ++lno_;
       return Context(buf_, end_, num_);
@@ -423,7 +432,7 @@ class AbstractMatcher {
     return Context(buf_, 0, num_);
   }
 #endif
-  /// Set buffer to 1 for interactive input.
+  /// Set interactive input with buffer size of 1 to read data bytewise which is very slow.
   void interactive()
     /// @note Use this method before any matching is done and before any input is read since the last time input was (re)set.
   {
@@ -436,6 +445,20 @@ class AbstractMatcher {
     DBGLOG("AbstractMatcher::flush()");
     pos_ = end_;
   }
+  /// Returns more input data directly from the source (method can be overriden, as by reflex::FlexLexer::get(s, n) for example that invokes reflex::FlexLexer::LexerInput(s, n)).
+  virtual size_t get(
+      /// @returns the nonzero number of (less or equal to n) 8-bit characters added to buffer s from the current input, or zero when EOF
+      char  *s, ///< points to the string buffer to fill with input
+      size_t n) ///< size of buffer pointed to by s
+  {
+    return in.get(s, n);
+  }
+  /// Returns true if wrapping of input after EOF is supported.
+  virtual bool wrap()
+    /// @returns true if input was succesfully wrapped
+  {
+    return false;
+  }
   /// Set the input character sequence for this matcher and reset/restart the matcher.
   virtual AbstractMatcher& input(const Input& input) ///< input character sequence for this matcher
     /// @returns this matcher
@@ -456,7 +479,11 @@ class AbstractMatcher {
       if (own_)
       {
 #if defined(WITH_REALLOC)
+#if (defined(__WIN32__) || defined(_WIN32) || defined(WIN32) || defined(_WIN64) || defined(__BORLANDC__)) && !defined(__CYGWIN__) && !defined(__MINGW32__) && !defined(__MINGW64__)
+        _aligned_free(static_cast<void*>(buf_));
+#else
         std::free(static_cast<void*>(buf_));
+#endif
 #else
         delete[] buf_;
 #endif
@@ -491,7 +518,7 @@ class AbstractMatcher {
   }
   
   /// Returns nonzero capture index (i.e. true) if the entire input matches this matcher's pattern (and internally caches the true/false result to permit repeat invocations).
-  size_t matches()
+  inline size_t matches()
     /// @returns nonzero capture index (i.e. true) if the entire input matched this matcher's pattern, zero (i.e. false) otherwise
   {
     if (!mat_ && at_bob())
@@ -499,25 +526,25 @@ class AbstractMatcher {
     return mat_;
   }
   /// Returns a positive integer (true) indicating the capture index of the matched text in the pattern or zero (false) for a mismatch.
-  size_t accept() const
+  inline size_t accept() const
     /// @returns nonzero capture index of the match in the pattern, which may be matcher dependent, or zero for a mismatch, or Const::EMPTY for the empty last split
   {
     return cap_;
   }
   /// Returns pointer to the begin of the matched text (non-0-terminated), a constant-time operation, use with end() or use size() for text end/length.
-  const char *begin() const
+  inline const char *begin() const
     /// @returns const char* pointer to the matched text in the buffer
   {
     return txt_;
   }
   /// Returns pointer to the exclusive end of the matched text, a constant-time operation.
-  const char *end() const
+  inline const char *end() const
     /// @returns const char* pointer to the exclusive end of the matched text in the buffer
   {
     return txt_ + len_;
   }
   /// Returns 0-terminated string of the text matched, does not include matched \0s, this is a constant-time operation.
-  const char *text()
+  inline const char *text()
     /// @returns 0-terminated const char* string with text matched
   {
     if (chr_ == '\0')
@@ -528,25 +555,25 @@ class AbstractMatcher {
     return txt_;
   }
   /// Returns the text matched as a string, a copy of text(), may include matched \0s.
-  std::string str() const
+  inline std::string str() const
     /// @returns string with text matched
   {
     return std::string(txt_, len_);
   }
   /// Returns the match as a wide string, converted from UTF-8 text(), may include matched \0s.
-  std::wstring wstr() const
+  inline std::wstring wstr() const
     /// @returns wide string with text matched
   {
     return wcs(txt_, len_);
   }
   /// Returns the length of the matched text in number of bytes, including matched \0s, a constant-time operation.
-  size_t size() const
+  inline size_t size() const
     /// @returns match size in bytes
   {
     return len_;
   }
   /// Returns the length of the matched text in number of wide characters.
-  size_t wsize() const
+  inline size_t wsize() const
     /// @returns the length of the match in number of wide (multibyte UTF-8) characters
   {
     size_t n = 0;
@@ -556,33 +583,150 @@ class AbstractMatcher {
     return n;
   }
   /// Returns the first 8-bit character of the text matched.
-  int chr() const
+  inline int chr() const
     /// @returns 8-bit char
   {
     return *txt_;
   }
   /// Returns the first wide character of the text matched.
-  int wchr() const
+  inline int wchr() const
     /// @returns wide char (UTF-8 converted to Unicode)
   {
     return utf8(txt_);
   }
+  /// Set or change the starting line number of the last match.
+  inline void lineno(size_t n) ///< new line number
+  {
+    if (lpb_ < txt_)
+      (void)lineno(); // update lno_ and bol_ (or cno_) before overriding lno_
+    lno_ = n;
+  }
   /// Updates and returns the starting line number of the match in the input character sequence.
-  size_t lineno()
+  inline size_t lineno()
     /// @returns line number
   {
 #if defined(WITH_SPAN)
     if (lpb_ < txt_)
     {
-      while (bol_ < txt_)
+      char *s = bol_;
+      char *t = txt_;
+      // clang/gcc 4-way vectorizable loop
+      while (t - 4 >= s)
       {
-        char *s = static_cast<char*>(std::memchr(bol_, '\n', txt_ - bol_));
-        if (s == NULL)
+        if ((t[-1] == '\n') | (t[-2] == '\n') | (t[-3] == '\n') | (t[-4] == '\n'))
           break;
-        ++lno_;
-        bol_ = s + 1;
+        t -= 4;
       }
+      // epilogue
+      if (--t >= s && *t != '\n')
+        if (--t >= s && *t != '\n')
+          if (--t >= s && *t != '\n')
+            --t;
+      bol_ = t + 1;
       lpb_ = txt_;
+      size_t n = lno_;
+#if defined(HAVE_AVX512BW) && (!defined(_MSC_VER) || defined(_WIN64))
+      if (have_HW_AVX512BW())
+      {
+        __m512i vlcn = _mm512_set1_epi8('\n');
+        while (s + 63 <= t)
+        {
+          __m512i vlcm = _mm512_loadu_si512(reinterpret_cast<const __m512i*>(s));
+          uint64_t mask = _mm512_cmpeq_epi8_mask(vlcm, vlcn);
+          n += popcountl(mask);
+          s += 64;
+        }
+      }
+      else if (have_HW_AVX2())
+      {
+        __m256i vlcn = _mm256_set1_epi8('\n');
+        while (s + 31 <= t)
+        {
+          __m256i vlcm = _mm256_loadu_si256(reinterpret_cast<const __m256i*>(s));
+          __m256i vlceq = _mm256_cmpeq_epi8(vlcm, vlcn);
+          uint32_t mask = _mm256_movemask_epi8(vlceq);
+          n += popcount(mask);
+          s += 32;
+        }
+      }
+      else if (have_HW_SSE2())
+      {
+        __m128i vlcn = _mm_set1_epi8('\n');
+        while (s + 15 <= t)
+        {
+          __m128i vlcm = _mm_loadu_si128(reinterpret_cast<const __m128i*>(s));
+          __m128i vlceq = _mm_cmpeq_epi8(vlcm, vlcn);
+          uint32_t mask = _mm_movemask_epi8(vlceq);
+          n += popcount(mask);
+          s += 16;
+        }
+      }
+#elif defined(HAVE_AVX2)
+      if (have_HW_AVX2())
+      {
+        __m256i vlcn = _mm256_set1_epi8('\n');
+        while (s + 31 <= t)
+        {
+          __m256i vlcm = _mm256_loadu_si256(reinterpret_cast<const __m256i*>(s));
+          __m256i vlceq = _mm256_cmpeq_epi8(vlcm, vlcn);
+          uint32_t mask = _mm256_movemask_epi8(vlceq);
+          n += popcount(mask);
+          s += 32;
+        }
+      }
+      else if (have_HW_SSE2())
+      {
+        __m128i vlcn = _mm_set1_epi8('\n');
+        while (s + 15 <= t)
+        {
+          __m128i vlcm = _mm_loadu_si128(reinterpret_cast<const __m128i*>(s));
+          __m128i vlceq = _mm_cmpeq_epi8(vlcm, vlcn);
+          uint32_t mask = _mm_movemask_epi8(vlceq);
+          n += popcount(mask);
+          s += 16;
+        }
+      }
+#elif defined(HAVE_SSE2)
+      if (have_HW_SSE2())
+      {
+        __m128i vlcn = _mm_set1_epi8('\n');
+        while (s + 15 <= t)
+        {
+          __m128i vlcm = _mm_loadu_si128(reinterpret_cast<const __m128i*>(s));
+          __m128i vlceq = _mm_cmpeq_epi8(vlcm, vlcn);
+          uint32_t mask = _mm_movemask_epi8(vlceq);
+          n += popcount(mask);
+          s += 16;
+        }
+      }
+#elif defined(HAVE_NEON)
+      {
+        // ARM AArch64/NEON SIMD optimized loop? - no code found yet that runs faster than the code below
+      }
+#endif
+      uint32_t n0 = 0, n1 = 0, n2 = 0, n3 = 0;
+      // clang/gcc 4-way vectorizable loop
+      while (s + 3 <= t)
+      {
+        n0 += s[0] == '\n';
+        n1 += s[1] == '\n';
+        n2 += s[2] == '\n';
+        n3 += s[3] == '\n';
+        s += 4;
+      }
+      n += n0 + n1 + n2 + n3;
+      // epilogue
+      if (s <= t)
+      {
+        n += *s == '\n';
+        if (++s <= t)
+        {
+          n += *s == '\n';
+          if (++s <= t)
+            n += *s == '\n';
+        }
+      }
+      lno_ = n;
     }
 #else
     size_t n = lno_;
@@ -615,7 +759,7 @@ class AbstractMatcher {
     return lno_;
   }
   /// Returns the number of lines that the match spans.
-  size_t lines()
+  inline size_t lines()
     /// @returns number of lines
   {
     size_t n = 1;
@@ -625,13 +769,13 @@ class AbstractMatcher {
     return n;
   }
   /// Returns the inclusive ending line number of the match in the input character sequence.
-  size_t lineno_end()
+  inline size_t lineno_end()
     /// @returns line number
   {
     return lineno() + lines() - 1;
   }
   /// Updates and returns the starting column number of the matched text, taking tab spacing into account and counting wide characters as one character each
-  size_t columno()
+  inline size_t columno()
     /// @returns column number
   {
     (void)lineno();
@@ -653,7 +797,7 @@ class AbstractMatcher {
 #endif
   }
   /// Returns the number of columns of the matched text, taking tab spacing into account and counting wide characters as one character each.
-  size_t columns()
+  inline size_t columns()
     /// @returns number of columns
   {
     // count columns in tabs and UTF-8 chars
@@ -700,7 +844,7 @@ class AbstractMatcher {
   }
 #if defined(WITH_SPAN)
   /// Returns the inclusive ending column number of the matched text on the ending matching line, taking tab spacing into account and counting wide characters as one character each
-  size_t columno_end()
+  inline size_t columno_end()
     /// @returns column number
   {
     if (len_ == 0)
@@ -955,6 +1099,13 @@ class AbstractMatcher {
     }
     return buf_ + end_;
   }
+  /// Returns the number of bytes in the buffer available to search from the current begin()/text() position.
+  size_t avail()
+  {
+    if (peek() == EOF)
+      return 0;
+    return end_ - (txt_ - buf_);
+  }
   /// Returns the byte offset of the match from the start of the line.
   size_t border()
     /// @returns border offset
@@ -1222,26 +1373,12 @@ class AbstractMatcher {
     own_ = false; // require allocation of a buffer
     reset(opt);
   }
-  /// Returns more input directly from the source (method can be overriden, as by reflex::FlexLexer::get(s, n) for example that invokes reflex::FlexLexer::LexerInput(s, n)).
-  virtual size_t get(
-      /// @returns the nonzero number of (less or equal to n) 8-bit characters added to buffer s from the current input, or zero when EOF
-      char  *s, ///< points to the string buffer to fill with input
-      size_t n) ///< size of buffer pointed to by s
-  {
-    return in.get(s, n);
-  }
-  /// Returns true if wrapping of input after EOF is supported.
-  virtual bool wrap()
-    /// @returns true if input was succesfully wrapped
-  {
-    return false;
-  }
   /// The abstract match operation implemented by pattern matching engines derived from AbstractMatcher.
   virtual size_t match(Method method)
     /// @returns nonzero when input matched the pattern using method Const::SCAN, Const::FIND, Const::SPLIT, or Const::MATCH
     = 0;
   /// Shift or expand the internal buffer when it is too small to accommodate more input, where the buffer size is doubled when needed, change cur_, pos_, end_, max_, ind_, buf_, bol_, lpb_, and txt_.
-  bool grow(size_t need = Const::BLOCK) ///< optional needed space = Const::BLOCK size by default
+  inline bool grow(size_t need = Const::BLOCK) ///< optional needed space = Const::BLOCK size by default
     /// @returns true if buffer was shifted or enlarged
   {
     if (max_ - end_ >= need + 1)
@@ -1277,7 +1414,11 @@ class AbstractMatcher {
         max_ *= 2;
       DBGLOG("Expand buffer to %zu bytes", max_);
 #if defined(WITH_REALLOC)
+#if (defined(__WIN32__) || defined(_WIN32) || defined(WIN32) || defined(_WIN64) || defined(__BORLANDC__)) && !defined(__CYGWIN__) && !defined(__MINGW32__) && !defined(__MINGW64__)
+      char *newbuf = static_cast<char*>(_aligned_realloc(static_cast<void*>(buf_), max_, 4096));
+#else
       char *newbuf = static_cast<char*>(std::realloc(static_cast<void*>(buf_), max_));
+#endif
       if (newbuf == NULL)
         throw std::bad_alloc();
 #else
@@ -1323,7 +1464,11 @@ class AbstractMatcher {
         num_ += gap;
 #if defined(WITH_REALLOC)
         std::memmove(buf_, txt_, end_);
+#if (defined(__WIN32__) || defined(_WIN32) || defined(WIN32) || defined(_WIN64) || defined(__BORLANDC__)) && !defined(__CYGWIN__) && !defined(__MINGW32__) && !defined(__MINGW64__)
+        char *newbuf = static_cast<char*>(_aligned_realloc(static_cast<void*>(buf_), max_, 4096));
+#else
         char *newbuf = static_cast<char*>(std::realloc(static_cast<void*>(buf_), max_));
+#endif
         if (newbuf == NULL)
           throw std::bad_alloc();
 #else
@@ -1394,7 +1539,7 @@ class AbstractMatcher {
     txt_ = buf_ + cur_;
   }
   /// Get the next character and grow the buffer to make more room if necessary.
-  int get_more()
+  inline int get_more()
     /// @returns the character read (unsigned char 0..255) or EOF (-1)
   {
     DBGLOG("AbstractMatcher::get_more()");
@@ -1416,7 +1561,7 @@ class AbstractMatcher {
     }
   }
   /// Peek at the next character and grow the buffer to make more room if necessary.
-  int peek_more()
+  inline int peek_more()
     /// @returns the character (unsigned char 0..255) or EOF (-1)
   {
     DBGLOG("AbstractMatcher::peek_more()");
