@@ -28,7 +28,7 @@
 
 /**
 @file      input.h
-@brief     RE/flex input character sequence class
+@brief     RE/flex input character sequence class and HW accelleration
 @author    Robert van Engelen - engelen@genivia.com
 @copyright (c) 2016-2020, Robert van Engelen, Genivia Inc. All rights reserved.
 @copyright (c) BSD-3 License - see LICENSE.txt
@@ -43,6 +43,24 @@
 #include <iostream>
 #include <string>
 
+#include <stdint.h>
+
+#if defined(HAVE_AVX512BW)
+# include <immintrin.h>
+#elif defined(HAVE_AVX2)
+# include <immintrin.h>
+#elif defined(HAVE_SSE2)
+# include <emmintrin.h>
+#elif defined(HAVE_NEON)
+# include <arm_neon.h>
+#endif
+
+#if defined(HAVE_AVX512BW) || defined(HAVE_AVX2) || defined(HAVE_SSE2)
+# ifdef _MSC_VER
+#  include <intrin.h>
+# endif
+#endif
+
 namespace reflex {
 
 /// Input character sequence class for unified access to sources of input text.
@@ -312,6 +330,8 @@ class Input {
     static const file_encoding_type koi8_ru    = 37; ///< KOI8-RU
     static const file_encoding_type custom     = 38; ///< custom code page
   };
+  /// FILE* handler functor base class to handle FILE* errors and non-blocking FILE* reads
+  struct Handler { virtual int operator()() = 0; };
   /// Stream buffer for reflex::Input, derived from std::streambuf.
   class streambuf;
   /// Stream buffer for reflex::Input to read DOS files, replaces CRLF by LF, derived from std::streambuf.
@@ -323,7 +343,8 @@ class Input {
       wstring_(NULL),
       file_(NULL),
       istream_(NULL),
-      size_(0)
+      size_(0),
+      handler_(NULL)
   {
     init();
   }
@@ -336,6 +357,7 @@ class Input {
       istream_(input.istream_),
       size_(input.size_),
       uidx_(input.uidx_),
+      ulen_(input.ulen_),
       utfx_(input.utfx_),
       page_(input.page_)
   {
@@ -443,7 +465,7 @@ class Input {
       istream_(NULL),
       size_(0)
   {
-    init();
+    init(enc);
     if (file_encoding() == file_encoding::plain)
       file_encoding(enc, page);
   }
@@ -478,6 +500,7 @@ class Input {
     istream_ = input.istream_;
     size_ = input.size_;
     uidx_ = input.uidx_;
+    ulen_ = input.ulen_;
     utfx_ = input.utfx_;
     page_ = input.page_;
     std::memcpy(utf8_, input.utf8_, sizeof(utf8_));
@@ -630,11 +653,9 @@ class Input {
     if (wstring_)
     {
       size_t k = n;
-      if (uidx_ < sizeof(utf8_))
+      if (ulen_ > 0)
       {
-        size_t l = 0;
-        while (uidx_ + l < sizeof(utf8_) && utf8_[uidx_ + l] != '\0')
-          ++l;
+        size_t l = ulen_;
         if (l > k)
           l = k;
         std::memcpy(s, utf8_ + uidx_, l);
@@ -642,12 +663,13 @@ class Input {
         if (k == 0)
         {
           uidx_ += static_cast<unsigned short>(l);
+          ulen_ -= static_cast<unsigned short>(l);
           if (size_ >= n)
             size_ -= n;
           return n;
         }
         s += l;
-        uidx_ = sizeof(utf8_);
+        ulen_ = 0;
       }
       wchar_t c;
       while ((c = *wstring_) != L'\0' && k > 0)
@@ -674,8 +696,8 @@ class Input {
           }
           if (k < l)
           {
-            utf8_[l] = '\0';
             uidx_ = static_cast<unsigned short>(k);
+            ulen_ = static_cast<unsigned short>(l);
             std::memcpy(s, utf8_, k);
             s += k;
             k = 0;
@@ -695,10 +717,12 @@ class Input {
     }
     if (file_)
     {
-      size_t k = file_get(s, n);
-      if (size_ >= k)
-        size_ -= k;
-      return k;
+      while (true)
+      {
+        size_t k = file_get(s, n);
+        if (k > 0 || feof(file_) || handler_ == NULL || (*handler_)() == 0)
+          return k;
+      }
     }
     if (istream_)
     {
@@ -721,17 +745,18 @@ class Input {
     return utfx_;
   }
   /// Initialize the state after (re)setting the input source, auto-detects UTF BOM in FILE* input if the file size is known.
-  void init()
+  void init(file_encoding_type enc = file_encoding::plain)
   {
     std::memset(utf8_, 0, sizeof(utf8_));
-    uidx_ = sizeof(utf8_);
+    uidx_ = 0;
+    ulen_ = 0;
     utfx_ = 0;
     page_ = NULL;
     if (file_ != NULL)
-      file_init();
+      file_init(enc);
   }
   /// Called by init() for a FILE*.
-  void file_init();
+  void file_init(file_encoding_type enc);
   /// Called by size() for a wstring.
   void wstring_size();
   /// Called by size() for a FILE*.
@@ -743,16 +768,23 @@ class Input {
       char  *s, ///< points to the string buffer to fill with input
       size_t n) ///< size of buffer pointed to by s
       ;
+  /// Set FILE* handler
+  void set_handler(Handler *handler)
+  {
+    handler_ = handler;
+  }
  protected:
   const char           *cstring_; ///< char string input (when non-null) of length reflex::Input::size_
   const wchar_t        *wstring_; ///< NUL-terminated wide string input (when non-null)
   FILE                 *file_;    ///< FILE* input (when non-null)
   std::istream         *istream_; ///< stream input (when non-null)
   size_t                size_;    ///< size of the remaining input in bytes (size_ == 0 may indicate size is not set)
-  char                  utf8_[8]; ///< UTF-8 normalization buffer
-  unsigned short        uidx_;    ///< index in utf8_[] or >= 8 when unused
+  char                  utf8_[8]; ///< UTF-8 normalization buffer, >=8 bytes
+  unsigned short        uidx_;    ///< index in utf8_[]
+  unsigned short        ulen_;    ///< length of data in utf8_[] or 0 if no data
   file_encoding_type    utfx_;    ///< file_encoding
   const unsigned short *page_;    ///< custom code page
+  Handler              *handler_; ///< to handle FILE* errors and non-blocking FILE* reads
 };
 
 /// Stream buffer for reflex::Input, derived from std::streambuf.
@@ -1138,6 +1170,71 @@ class BufferedInput::dos_streambuf : public std::streambuf {
   int ch2_;
 };
 
+#if defined(HAVE_AVX512BW) || defined(HAVE_AVX2) || defined(HAVE_SSE2)
+
+#ifdef _MSC_VER
+#pragma intrinsic(_BitScanForward)
+inline uint32_t ctz(uint32_t x)
+{
+  unsigned long r;
+  _BitScanForward(&r, x);
+  return r;
+}
+inline uint32_t popcount(uint32_t x)
+{
+  return __popcnt(x);
+}
+#ifdef _WIN64
+#pragma intrinsic(_BitScanForward64)
+inline uint32_t ctzl(uint64_t x)
+{
+  unsigned long r;
+  _BitScanForward64(&r, x);
+  return r;
+}
+inline uint32_t popcountl(uint64_t x)
+{
+  return static_cast<uint32_t>(__popcnt64(x));
+}
+#endif
+#else
+inline uint32_t ctz(uint32_t x)
+{
+  return __builtin_ctz(x);
+}
+inline uint32_t ctzl(uint64_t x)
+{
+  return __builtin_ctzl(x);
+}
+inline uint32_t popcount(uint32_t x)
+{
+  return __builtin_popcount(x);
+}
+inline uint32_t popcountl(uint64_t x)
+{
+  return __builtin_popcountl(x);
+}
+#endif
+
+#endif
+
+extern uint64_t HW;
+
+inline bool have_HW_AVX512BW()
+{
+  return HW & (1ULL << 62);
+}
+
+inline bool have_HW_AVX2()
+{
+  return HW & (1ULL << 37);
+}
+
+inline bool have_HW_SSE2()
+{
+  return HW & (1ULL << 26);
+}
+
 } // namespace reflex
 
 #endif
