@@ -74,6 +74,8 @@
       DBGLOGA("!"); \
     if ((p).ticked()) \
       DBGLOGA("'"); \
+    if ((p).negate()) \
+      DBGLOGA("-"); \
   }
 #endif
 
@@ -139,7 +141,7 @@ const std::string Pattern::operator[](Accept choice) const
 {
   if (choice == 0)
     return rex_;
-  if (choice >= 1 && choice <= size())
+  if (choice <= size())
   {
     Location loc = end_.at(choice - 1);
     Location prev = 0;
@@ -150,6 +152,15 @@ const std::string Pattern::operator[](Accept choice) const
   return "";
 }
 
+void Pattern::error(regex_error_type code, size_t pos) const
+{
+  regex_error err(code, rex_, pos);
+  if (opt_.w)
+    std::cerr << err.what();
+  if (code == regex_error::exceeds_limits || opt_.r)
+    throw err;
+}
+
 void Pattern::init(const char *options, const uint8_t *pred)
 {
   init_options(options);
@@ -710,8 +721,8 @@ void Pattern::parse3(
       if (at(++loc) == '?')
       {
         if (++lazyidx == 0)
-          error(regex_error::exceeds_limits, loc);
-        lazyset.insert(lazyidx); // overflow: exceeds max 255 lazy quantifiers
+          error(regex_error::exceeds_limits, loc); // overflow: exceeds max 255 lazy quantifiers
+        lazyset.insert(lazyidx);
         if (nullable)
           lazy(lazyset, firstpos);
         ++loc;
@@ -773,9 +784,7 @@ void Pattern::parse3(
             error(regex_error::exceeds_limits, loc); // overflow: exceeds max 255 lazy quantifiers
           lazyset.insert(lazyidx);
           if (nullable)
-          {
             lazy(lazyset, firstpos);
-          }
           /* CHECKED algorithmic options: 8/1 else
              {
              lazy(lazyset, firstpos, firstpos1);
@@ -894,13 +903,14 @@ void Pattern::parse4(
         if (c == ')')
           ++loc;
       }
-      else if (c == '^') // (?^ negative pattern to be ignored (new mode)
+      else if (c == '^') // (?^ negative pattern to be ignored (new mode), producing a redo match
       {
+        Positions firstpos1;
         ++loc;
         parse1(
             begin,
             loc,
-            firstpos,
+            firstpos1,
             lastpos,
             nullable,
             followpos,
@@ -909,8 +919,8 @@ void Pattern::parse4(
             modifiers,
             lookahead,
             iter);
-        for (Positions::const_iterator p = lastpos.begin(); p != lastpos.end(); ++p)
-          followpos[p->pos()].insert(Position(0).accept(true));
+        for (Positions::iterator p = firstpos1.begin(); p != firstpos1.end(); ++p)
+          firstpos.insert(p->negate(true));
       }
       else if (c == '=') // (?= lookahead
       {
@@ -1443,6 +1453,12 @@ void Pattern::compile(
         }
         Char lo = i->first.lo();
         Char max = i->first.hi();
+#ifdef DEBUG
+        DBGLOGN("from state %p on %02x-%02x move to {", state, lo, max);
+        for (Positions::const_iterator p = pos.begin(); p != pos.end(); ++p)
+          DBGLOGPOS(*p);
+        DBGLOGN(" } = state %p", target_state);
+#endif
         while (lo <= max)
         {
           if (i->first.contains(lo))
@@ -1557,11 +1573,11 @@ void Pattern::trim_lazy(Positions *pos) const
     pos->erase(--p.base());
   }
 #endif
-  // trims accept positions keeping the first only, and keeping redo (positions with accept == 0)
+  // trims accept positions keeping the first only
   Positions::iterator q = pos->begin(), a = pos->end();
   while (q != pos->end())
   {
-    if (q->accept() && q->accepts() != 0)
+    if (q->accept() && !q->negate())
     {
       if (a == pos->end())
         a = q++;
@@ -1596,8 +1612,8 @@ void Pattern::compile_transition(
     {
       Accept accept = k->accepts();
       if (state->accept == 0 || accept < state->accept)
-        state->accept = accept; // pick lowest nonzero accept index
-      if (accept == 0)
+        state->accept = accept;
+      if (k->negate())
         state->redo = true;
     }
     else
@@ -1655,9 +1671,20 @@ void Pattern::compile_transition(
       }
       else
       {
-        Follow::const_iterator i = followpos.find(k->pos());
+        Follow::iterator i = followpos.find(k->pos());
         if (i != followpos.end())
         {
+          if (k->negate())
+          {
+            Positions::const_iterator b = i->second.begin();
+            if (b != i->second.end() && !b->negate())
+            {
+              Positions to;
+              for (Positions::const_iterator p = b; p != i->second.end(); ++p)
+                to.insert(p->negate(true));
+              i->second.swap(to);
+            }
+          }
           if (k->lazy())
           {
 #if 1 // CHECKED algorithmic options: 7/31 this optimization works fine when trim_lazy adds non-lazy greedy state, but may increase the total number of states:
@@ -1806,7 +1833,7 @@ void Pattern::transition(
   Moves::iterator end = moves.end();
   while (i != end)
   {
-    if (is_subset(i->second, follow))
+    if (i->second == follow)
     {
       chars += i->first;
       moves.erase(i++);
@@ -2736,6 +2763,8 @@ void Pattern::export_dfa(const DFA::State *start) const
               ::fprintf(file, "!");
             if (i->ticked())
               ::fprintf(file, "'");
+            if (i->negate())
+              ::fprintf(file, "-");
             if (k++ % n)
               sep = " ";
             else
@@ -2744,7 +2773,7 @@ void Pattern::export_dfa(const DFA::State *start) const
           if ((state->accept && !state->redo) || !state->heads.empty() || !state->tails.empty())
             ::fprintf(file, "\\n");
 #endif
-          if (state->accept && !state->redo)
+          if (state->accept > 0 && !state->redo)
             ::fprintf(file, "[%u]", state->accept);
           for (Lookaheads::const_iterator i = state->tails.begin(); i != state->tails.end(); ++i)
             ::fprintf(file, "%u>", *i);
@@ -2752,7 +2781,7 @@ void Pattern::export_dfa(const DFA::State *start) const
             ::fprintf(file, "<%u", *i);
           if (state->redo)
             ::fprintf(file, "\",style=dashed,peripheries=1];\n");
-          else if (state->accept)
+          else if (state->accept > 0)
             ::fprintf(file, "\",peripheries=2];\n");
           else if (!state->heads.empty())
             ::fprintf(file, "\",style=dashed,peripheries=2];\n");
@@ -2957,7 +2986,7 @@ void Pattern::predict_match_dfa(DFA::State *start)
     }
     state = next;
   }
-  if (state != NULL && state->accept != 0 && !state->edges.empty())
+  if (state != NULL && state->accept > 0 && !state->edges.empty())
     one_ = false;
   min_ = 0;
   std::memset(bit_, 0xFF, sizeof(bit_));
@@ -3026,7 +3055,7 @@ void Pattern::gen_predict_match_transitions(DFA::State *state, std::map<DFA::Sta
       break;
     }
     DFA::State *next = edge->second.second;
-    bool accept = (next == NULL || next->accept != 0);
+    bool accept = (next == NULL || next->accept > 0);
     if (!accept)
     {
       for (DFA::State::Edges::const_iterator e = next->edges.begin(); e != next->edges.end(); ++e)
@@ -3069,7 +3098,7 @@ void Pattern::gen_predict_match_transitions(size_t level, DFA::State *state, ORa
     if (is_meta(lo))
       break;
     DFA::State *next = level < 7 ? edge->second.second : NULL;
-    bool accept = next == NULL || next->accept != 0;
+    bool accept = next == NULL || next->accept > 0;
     if (!accept)
     {
       for (DFA::State::Edges::const_iterator e = next->edges.begin(); e != next->edges.end(); ++e)
