@@ -18,8 +18,8 @@ cat >> man/ugrep.1 << 'END'
 \fBugrep\fR, \fBug\fR -- file pattern searcher
 .SH SYNOPSIS
 .B ugrep
-[\fIOPTIONS\fR] [\fB-A\fR \fINUM\fR] [\fB-B\fR \fINUM\fR] [\fB-C\fR[\fINUM\fR]] [\fIPATTERN\fR] [\fB-f\fR \fIFILE\fR]
-      [\fB-e\fR \fIPATTERN\fR] [\fB-N\fR \fIPATTERN\fR] [\fB-t\fR \fITYPES\fR] [\fB-J\fR [\fINUM\fR]] [\fB--sort\fR[=\fIKEY\fR]]
+[\fIOPTIONS\fR] [\fB-A\fR \fINUM\fR] [\fB-B\fR \fINUM\fR] [\fB-C \fR\fINUM\fR] [\fB-y\fR] [\fB-Q\fR|\fIPATTERN\fR] [\fB-f\fR \fIFILE\fR]
+      [\fB-e\fR \fIPATTERN\fR] [\fB-N\fR \fIPATTERN\fR] [\fB-t\fR \fITYPES\fR] [\fB-g\fR \fIGLOBS\fR] [\fB--sort\fR[=\fIKEY\fR]]
       [\fB--color\fR[=\fIWHEN\fR]|\fB--colour\fR[=\fIWHEN\fR]] [\fB--pager\fR[=\fICOMMAND\fR]] [\fIFILE\fR \fI...\fR]
 .SH DESCRIPTION
 The \fBugrep\fR utility searches any given input files, selecting lines that
@@ -35,6 +35,9 @@ automatically normalized; otherwise, \fBugrep\fR assumes the input is ASCII,
 UTF-8, or raw binary.  An input encoding format may be specified with option
 \fB--encoding\fR.
 .PP
+The \fBug\fR command is equivalent to \fBugrep --config\fR to load the default
+configuration file, which allows for customization, see CONFIGURATION.
+.PP
 If no \fIFILE\fR arguments are specified and standard input is read from a
 terminal, recursive searches are performed as if \fB-R\fR is specified.  To
 force reading from standard input, specify `-' as a \fIFILE\fR argument.
@@ -47,8 +50,11 @@ Hidden files and directories are ignored in recursive searches.  Option
 \fB-.\fR (\fB--hidden\fR) includes hidden files and directories in recursive
 searches.
 .PP
-The \fBug\fR command is equivalent to \fBugrep --config\fR to load the default
-configuration file, which allows for customization, see CONFIGURATION.
+A query interface is opened with \fB-Q\fR (\fB--query\fR) to interactively
+specify search patterns and view search results.  Note that a \fIPATTERN\fR
+argument should not be specified in this case.
+.PP
+For help, \fB--help \fIWHAT\fR displays help on options related to \fIWHAT\fR.
 .PP
 The following options are available:
 END
@@ -132,6 +138,8 @@ The configuration is written to standard output when \fIFILE\fR is a `-'.
 Globbing is used by options \fB-g\fR, \fB--include\fR, \fB--include-dir\fR,
 \fB--include-from\fR, \fB--exclude\fR, \fB--exclude-dir\fR,
 \fB--exclude-from\fR to match pathnames and basenames in recursive searches.
+Glob arguments for these options should be quoted to prevent shell globbing.
+.PP
 Globbing supports gitignore syntax and the corresponding matching rules.  When
 a glob ends in a path separator it matches directories as if
 \fB--include-dir\fR or \fB--exclude-dir\fR is specified.  When a glob contains
@@ -197,6 +205,17 @@ Matches a/b, a/x/b, a/x/y/b, but not x/a/b, a/b/x
 Matches a/x, a/y, a/x/y,     but not a, b/x
 .IP \fBa\\\\?b\fR
 Matches a?b,                 but not a, b, ab, axb, a/b
+.PP
+Note that exclude glob patterns take priority over include glob patterns when
+specified with options -g, --exclude, --exclude-dir, --include and include-dir.
+.PP
+Glob patterns specified with prefix `!' in any of the files associated with
+--include-from, --exclude-from and --ignore-files will negate a previous glob
+match.  That is, any matching file or directory excluded by a previous glob
+pattern specified in the files associated with --exclude-from or --ignore-file
+will become included again.  Likewise, any matching file or directory included
+by a previous glob pattern specified in the files associated with
+--include-from will become excluded again.
 .SH ENVIRONMENT
 .IP \fBGREP_PATH\fR
 May be used to specify a file path to pattern files.  The file path is used by
@@ -256,6 +275,10 @@ Fields may be used in \fIFORMAT\fR, which expand into the following values:
 if option \fB-H\fR is used: \fIARG\fR, the file pathname and separator.
 .IP \fB%f\fR
 the file pathname.
+.IP \fB%a\fR
+the file basename without directory path.
+.IP \fB%p\fR
+the directory path to the file.
 .IP \fB%z\fR
 the file pathname in a (compressed) archive.
 .IP \fB%[\fR\fIARG\fR\fB]H\fR
@@ -340,11 +363,11 @@ same as \fB%[1]#\fR; requires option \fB-P\fR.
 .IP \fB%[\fINUM\fR\fB]#\fR
 the regex group capture \fINUM\fR; requires option \fB-P\fR.
 .IP \fB%G\fR
-list of group capture indices/names of the match (option -P).
+list of group capture indices/names of the match (option \fB-P\fR).
 .IP \fB%[NAME1|NAME2|...]G\fR
 NAMEs corresponding to the group capture indices of the match.
 .IP \fB%g\fR
-the group capture index/name of the match or 1 (option -P).
+the group capture index/name of the match or 1 (option \fB-P\fR).
 .IP \fB%[NAME1|NAME2|...]g\fR
 NAME corresponding to the group capture index of the match.
 .IP \fB%%\fR
@@ -365,9 +388,9 @@ reverts the separator to the default separator or the separator specified with
 .PP
 Formatted output is written for each matching pattern, which means that a line
 may be output multiple times when patterns match more than once on the same
-line.  If field \fB%u\fR is specified anywhere in a format string, then
-matching lines are output only once unless option \fB-u\fR, \fB--ungroup\fR is
-specified or when one or more newlines are matched.
+line.  If field \fB%u\fR is specified anywhere in a format string, matching
+lines are output only once, unless option \fB-u\fR, \fB--ungroup\fR is
+specified or when more than one line of input matched the search pattern.
 .PP
 Additional formatting options:
 .IP \fB--format-begin\fR=\fIFORMAT\fR
@@ -379,9 +402,9 @@ the \fIFORMAT\fR when closing a file and a match was found.
 .IP \fB--format-end\fR=\fIFORMAT\fR
 the \fIFORMAT\fR when ending the search.
 .PP
-The context options \fB-A\fR, \fB-B\fR, \fB-C\fR, \fB-y\fR, and options
-\fB-v\fR, \fB--break\fR, \fB--heading\fR, \fB--color\fR, \fB-T\fR, and
-\fB--null\fR have no effect on formatted output.
+The context options \fB-A\fR, \fB-B\fR, \fB-C\fR, \fB-y\fR, and display options
+\fB--break\fR, \fB--heading\fR, \fB--color\fR, \fB-T\fR, and \fB--null\fR have
+no effect on formatted output.
 .SH EXAMPLES
 Display lines containing the word `patricia' in `myfile.txt':
 .IP
@@ -404,6 +427,14 @@ Count the number of words `patricia', ignoring case:
 .IP
 $ ugrep -cowi patricia myfile.txt
 .PP
+List lines with both `amount' and a decimal number, ignoring case:
+.IP
+$ ugrep -wi --bool 'amount \d+(\.\d+)?' myfile.txt
+.PP
+Alternative query:
+.IP
+$ ugrep -wi -e amount --and '\d+(\.\d+)?' myfile.txt
+.PP
 List all Unicode words in a file:
 .IP
 $ ugrep -o '\\w+' myfile.txt
@@ -459,14 +490,15 @@ Output a list of line numbers of lines with `FIXME' but not `later':
 $ ugrep -n FIXME myfile.cpp | ugrep -vw later | 
   ugrep -P '^(\\d+)' --format='%,%n'
 .PP
-Monitor the system log for bug reports:
-.IP
-$ tail -f /var/log/system.log | ugrep -iw bug
-.PP
 Find lines with `FIXME' in the C/C++ files stored in a tarball:
 .IP
 $ ugrep -z -tc++ -n FIXME project.tgz
 .PP
+Recursively find lines with `FIXME' in C/C++ files, but do not search any `bak'
+and `old' directories:
+.IP
+$ ugrep -n FIXME -tc++ -g^bak/,^old/
+.PP
 Recursively search for the word `copyright' in cpio/jar/pax/tar/zip archives,
 compressed and regular files, and in PDFs using a PDF filter:
 .IP
@@ -488,13 +520,25 @@ List all files that are not ignored by one or more `.gitignore':
 $ ugrep -l '' --ignore-files
 .PP
 List all files containing a RPM signature, located in the `rpm' directory and
-recursively below up to two levels deeper (3 levels):
+recursively below up to two levels deeper (3 levels total):
 .IP
 $ ugrep -3 -l -tRpm '' rpm/
 .PP
+Monitor the system log for bug reports and ungroup multiple matches on a line:
+.IP
+$ tail -f /var/log/system.log | ugrep -u -i -w bug
+.PP
+Interactive fuzzy search with Boolean search queries:
+.IP
+$ ugrep -Q --bool -Z3 --sort=best
+.PP
 Display all words in a MacRoman-encoded file that has CR newlines:
 .IP
 $ ugrep --encoding=MACROMAN '\\w+' mac.txt
+.PP
+Display all options related to "fuzzy" searching:
+.IP
+$ ugrep --help fuzzy
 .SH BUGS
 Report bugs at:
 .IP
